import _objectWithoutProperties from '@babel/runtime/helpers/esm/objectWithoutProperties';
import _extends from '@babel/runtime/helpers/esm/extends';
import { Fragment, forwardRef } from 'react';
import { useId, jsx, useTheme, Portal } from '@keystone-ui/core';
import { usePopover } from '@keystone-ui/popover';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

const Tooltip = ({
  children,
  content,
  placement = 'top'
}) => {
  const {
    isOpen,
    setOpen,
    trigger,
    dialog,
    arrow
  } = usePopover({
    placement,
    modifiers: [{
      name: 'offset',
      options: {
        offset: [0, 12]
      }
    }]
  });
  const tooltipId = useId();

  const showTooltip = () => setOpen(true);

  const hideTooltip = () => setOpen(false);

  return jsx(Fragment, null, children({
    onMouseEnter: showTooltip,
    onMouseLeave: hideTooltip,
    onFocus: showTooltip,
    onBlur: hideTooltip,
    'aria-describedby': tooltipId,
    ref: trigger.ref
  }), jsx(TooltipElement, _extends({
    id: tooltipId,
    isVisible: isOpen,
    ref: dialog.ref
  }, dialog.props, {
    arrow: arrow
  }), content));
}; // Styled Component
// ------------------------------

const TooltipElement = /*#__PURE__*/forwardRef((_ref, consumerRef) => {
  let {
    isVisible,
    children,
    arrow
  } = _ref,
      props = _objectWithoutProperties(_ref, ["isVisible", "children", "arrow"]);

  const {
    elevation,
    radii,
    colors,
    shadow,
    spacing,
    typography
  } = useTheme();
  const arrowStyles = useArrowStyles();
  return jsx(Portal, null, jsx("div", _extends({
    role: "tooltip",
    "aria-hidden": !isVisible,
    ref: consumerRef,
    css: _objectSpread2({
      backgroundColor: colors.foregroundDim,
      borderRadius: radii.small,
      boxShadow: shadow.s200,
      color: colors.background,
      fontWeight: typography.fontWeight.medium,
      lineHeight: typography.leading.base,
      maxWidth: 320,
      // less than desirable magic number, but not sure if this needs to be in theme...
      opacity: isVisible ? 1 : 0,
      padding: `${spacing.small}px ${spacing.medium}px`,
      pointerEvents: isVisible ? undefined : 'none',
      zIndex: elevation.e500
    }, arrowStyles)
  }, props), children, jsx("div", _extends({
    "data-popper-arrow": true,
    ref: arrow.ref,
    className: "tooltipArrow"
  }, arrow.props))));
});

const useArrowStyles = () => {
  const {
    colors,
    shadow
  } = useTheme();
  return {
    '.tooltipArrow': {
      position: 'absolute',
      overflow: 'hidden',
      pointerEvents: 'none',
      height: '20px',
      width: '20px',
      '&::after': {
        content: 'close-quote',
        position: 'absolute',
        width: '10px',
        height: '10px',
        backgroundColor: colors.foregroundDim,
        transform: 'translateX(-50%) translateY(-50%) rotate(45deg)',
        boxShadow: shadow.s200
      }
    },
    "&[data-popper-placement^='left'] > .tooltipArrow": {
      left: '100%',
      '&::after': {
        top: '50%',
        left: '0'
      }
    },
    "&[data-popper-placement^='right'] > .tooltipArrow": {
      right: '100%',
      '&::after': {
        top: '50%',
        left: '100%'
      }
    },
    "&[data-popper-placement^='top'] > .tooltipArrow": {
      top: '100%',
      '&::after': {
        top: 0,
        bottom: '-50%',
        left: '50%'
      }
    },
    "&[data-popper-placement^='bottom'] > .tooltipArrow": {
      bottom: '100%',
      right: 'unset',
      '&::after': {
        bottom: '-50%',
        left: '50%'
      }
    }
  };
};

export { Tooltip };
